/* libcm - A library with a xlib wrapper and a gl based scene graph
 * Copyright (C) 2005, 2006  Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 * Author: Soren Sandmann (sandmann@redhat.com)
 */
#include "magnifier.h"
#include "state.h"
#include <math.h>

G_DEFINE_TYPE (CmMagnifier, cm_magnifier, CM_TYPE_NODE);

static void cm_magnifier_render (CmNode *node,
				 CmState *state);

static void
cm_magnifier_finalize (GObject *object)
{
    G_OBJECT_CLASS (cm_magnifier_parent_class)->finalize (object);
}

static void
cm_magnifier_render (CmNode *node,
		     CmState *state)
{
    CmMagnifier *mag = CM_MAGNIFIER (node);
    
    cm_node_render (mag->child, state);

#if 0
    glScissor (mag->target.x,
	       mag->target.y,
	       mag->target.height,
	       mag->target.width);

    glEnable (GL_SCISSOR_TEST);
#endif

    glPushMatrix();

#if 0
    glColor4f (0.0, 0.0, 1.0, 1.0);

    static GTimer *timer;
    if (!timer)
	timer = g_timer_new ();
    double t = 0.5 * g_timer_elapsed (timer, NULL);
    
    glTranslatef (-500, -300, 0.0);
    glScalef (sin (t), cos(t), 1.0);
#endif
    
    cm_node_render (mag->child, state);

    glPopMatrix();
    
#if 0
    glDisable (GL_SCISSOR_TEST);
#endif
}

static void
cm_magnifier_class_init (CmMagnifierClass *class)
{
    GObjectClass *object_class = G_OBJECT_CLASS (class);
    CmNodeClass *node_class = CM_NODE_CLASS (class);
    
    object_class->finalize = cm_magnifier_finalize;
    node_class->render = cm_magnifier_render;
}

static void
cm_magnifier_init (CmMagnifier *magnifier)
{
}

CmMagnifier *
cm_magnifier_new (CmNode *child,
		  WsRectangle *source,
		  WsRectangle *target)
{
    g_return_val_if_fail (child != NULL, NULL);
    CmMagnifier *mag = g_object_new (CM_TYPE_MAGNIFIER, NULL);

    mag->source = *source;
    mag->target = *target;

    cm_node_own_child (CM_NODE (mag), &mag->child, child);
    
    return mag;
}
