/* libcm - A library with a xlib wrapper and a gl based scene graph
 * Copyright (C) 2005, 2006  Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 * Author: Soren Sandmann (sandmann@redhat.com)
 */
#include "square.h"
#include "node.h"
#include "state.h"

static void cm_square_render (CmNode *node, CmState *state);

G_DEFINE_TYPE (CmSquare, cm_square, CM_TYPE_NODE);

static void
cm_square_finalize (GObject *object)
{
    G_OBJECT_CLASS (cm_square_parent_class)->finalize (object);
}

static void
cm_square_class_init (CmSquareClass *class)
{
    GObjectClass *object_class = G_OBJECT_CLASS (class);
    CmNodeClass *node_class = CM_NODE_CLASS (class);
    object_class->finalize = cm_square_finalize;
    node_class->render = cm_square_render;
}

static void
cm_square_render (CmNode *node,
		  CmState *state)
{
    CmSquare *square = CM_SQUARE (node);
    
    glColor4f (square->r, square->g, square->b, square->a);

    glPushMatrix ();

    glScalef (square->scale, square->scale, 1.0);
    
    glBegin (GL_QUADS);
    glVertex2f (-1.0, -1.0);
    glVertex2f (-1.0, 1.0);
    glVertex2f (1.0, 1.0);
    glVertex2f (1.0, -1.0);
    glEnd ();

    glPopMatrix();
}

static void
cm_square_init (CmSquare *square)
{
    
}

CmSquare *
cm_square_new (gdouble r,
	       gdouble g,
	       gdouble b,
	       gdouble a)
{
    CmSquare *square = g_object_new (CM_TYPE_SQUARE, NULL);

    square->r = r;
    square->g = g;
    square->b = b;
    square->a = a;
    
    square->scale = 1.0;

    return square;
}

void
cm_square_set_scale (CmSquare *square,
		     gdouble scale)
{
    square->scale = scale;

    cm_node_queue_repaint (square);
}
